package compbio.stat.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;

import org.apache.log4j.Logger;

import compbio.stat.servlet.util.Totals;
import compbio.ws.client.Services;
import compbio.ws.client.WSTester;

/**
 * Use cases:
 * <dl>
 * <li>Test web services and display results on the web page</li>
 * </dl>
 * 
 * @author pvtroshin
 * 
 */
public class ServiceStatus extends HttpServlet {

	private final static Logger log = Logger.getLogger(ServiceStatus.class);


	private static String getURL(HttpServletRequest req) {
		String scheme = req.getScheme();             // http
		String serverName = req.getServerName();     // hostname.com
		int serverPort = req.getServerPort();        // 80
		String contextPath = req.getContextPath();   // /mywebapp
		String servletPath = req.getServletPath();   // /servlet/MyServlet
		String pathInfo = req.getPathInfo();         // /a/b;c=123
		String queryString = req.getQueryString();          // d=789

		// Reconstruct original requesting URL
		StringBuffer url =  new StringBuffer();
		url.append(scheme).append("://").append(serverName);

		if ((serverPort != 80) && (serverPort != 443)) {
			url.append(":").append(serverPort);
		}

		url.append(contextPath).append(servletPath);

		if (pathInfo != null) {
			url.append(pathInfo);
		}
		if (queryString != null) {
			url.append("?").append(queryString);
		}
		return url.toString();
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		StringBuffer jabawspath = req.getRequestURL();
		jabawspath = jabawspath.delete(jabawspath.lastIndexOf("/"), jabawspath.length());
		String serverPath = jabawspath.toString();
/**
		String host = java.net.InetAddress.getLocalHost().getCanonicalHostName();
		System.out.println("Testing services at " + req.getRequestURL()); 
		System.out.println("Really: Testing services at " + serverPath); 
		System.out.println("      serverPath2 is " + getURL(req)); 
		System.out.println("      Context Path is " + req.getContextPath()); 
		System.out.println("      Host is " + host); 
		System.out.println("      Request URI is " + req.getRequestURI());
		System.out.println("      Servlet Path is " + req.getServletPath());
		System.out.println("      Path Info is " + req.getPathInfo());
		System.out.println("      Local Port is " + req.getLocalPort());
		System.out.println("      Remote Port is " + req.getRemotePort());
		System.out.println("      Local address is " + req.getLocalAddr());
		System.out.println("      Remote address is " + req.getRemoteAddr());
		@SuppressWarnings("unchecked")
		List<String> list = Collections.list(req.getHeaderNames());
		System.out.println("      HTTP header names are " + list.toString());
		for (String hdr : list) {
			System.out.println("      " + hdr + " header is " + req.getHeader(hdr));
		}
*/
		List<ServiceTestResult> testResults = new ArrayList<ServiceTestResult>();
		long startTime = System.nanoTime();
		for (Services service : Services.values()) {
			StringWriter testres = new StringWriter();
			PrintWriter writer = new PrintWriter(testres, true);
			WSTester tester = new WSTester(serverPath, writer);
			ServiceTestResult result = new ServiceTestResult(service);
			try {
				result.failed = tester.checkService(service);
			} catch (Exception e) {
				log.info(e, e.getCause());
				String mess = "Fails to connect to the web service: " + service + ". Reason: ";
				writer.println(mess + e.getLocalizedMessage() + "\nDetails: ");
				e.printStackTrace(writer);
			} finally {
				writer.close();
			}
			result.details = testres.toString();
			testResults.add(result);
		}
		req.setAttribute("results", testResults);
		long endTime = System.nanoTime();
		req.setAttribute("timeexec", (endTime - startTime) / 1000000);
		RequestDispatcher rd = req.getRequestDispatcher("statpages/ServicesStatus.jsp");
		rd.forward(req, resp);
	}

}
